@@ -43,6 +43,11 @@ pub trait Parker: Send + Clone {
     /// Maximum number of threads supported by this parker implementation.
     fn max_threads(&self) -> Option<usize>;
 
+    /// Set thread state to `Awake`, no matter what the current state is.
+    ///
+    /// This is mostly meant for cleanup after a thread panic, where the state is unkown.
+    fn init(&self, thread_id: usize) -> ();
+
     /// Prepare to go to sleep.
     /// You *must* call this before doing one last check on the resource being managed.
     /// If the resource is available afterwards, you *must* call [`abort_park`].
@@ -92,6 +97,11 @@ impl Parker for DynParker {
         self.inner.max_threads()
     }
 
+    #[inline(always)]
+    fn init(&self, thread_id: usize) -> () {
+        self.inner.init(thread_id);
+    }
+
     #[inline(always)]
     fn prepare_park(&self, thread_id: usize) -> () {
         self.inner.prepare_park(thread_id);
@@ -169,6 +179,11 @@ where
         self.inner.max_threads()
     }
 
+    #[inline(always)]
+    fn init(&self, thread_id: usize) -> () {
+        self.inner.init(thread_id);
+    }
+
     #[inline(always)]
     fn prepare_park(&self, thread_id: usize) -> () {
         self.inner.prepare_park(thread_id);
@@ -203,6 +218,9 @@ pub trait ThreadData: std::fmt::Debug {
     /// Maximum number of threads supported by this parker implementation.
     fn max_threads(&self) -> Option<usize>;
 
+    /// Initialise thread at `thread_id`.
+    fn init(&self, thread_id: usize) -> ();
+
     /// Prepare to go to sleep.
     fn prepare_park(&self, thread_id: usize) -> ();
 
@@ -251,6 +269,15 @@ impl ThreadData for SmallThreadData {
         Some(SmallThreadData::MAX_THREADS)
     }
 
+    fn init(&self, thread_id: usize) -> () {
+        assert!(thread_id < 32);
+        if let Ok(mut guard) = self.sleeping.lock() {
+            guard[thread_id] = ParkState::Awake;
+        } else {
+            panic!("Mutex is poisoned!");
+        }
+    }
+
     fn prepare_park(&self, thread_id: usize) -> () {
         assert!(thread_id < 32);
         self.sleep_set.set_at(thread_id);
@@ -387,6 +414,15 @@ impl ThreadData for LargeThreadData {
         Some(LargeThreadData::MAX_THREADS)
     }
 
+    fn init(&self, thread_id: usize) -> () {
+        assert!(thread_id < 64);
+        if let Ok(mut guard) = self.sleeping.lock() {
+            guard[thread_id] = ParkState::Awake;
+        } else {
+            panic!("Mutex is poisoned!");
+        }
+    }
+
     fn prepare_park(&self, thread_id: usize) -> () {
         assert!(thread_id < 64);
         self.sleep_set.set_at(thread_id);
@@ -527,6 +563,15 @@ impl ThreadData for DynamicThreadData {
         None
     }
 
+    fn init(&self, thread_id: usize) -> () {
+        if let Ok(mut guard) = self.data.lock() {
+            // Doesn't matter if it was there or not
+            let _ = guard.sleeping.remove(&thread_id);
+        } else {
+            panic!("Mutex is poisoned!");
+        }
+    }
+
     fn prepare_park(&self, _thread_id: usize) -> () {
         self.sleep_count.fetch_add(1usize, Ordering::SeqCst);
     }
